import { forEach, get, reduce, union } from 'lodash';
import chroma from 'chroma-js';
let staticOptionsCache;
let urlOptionsCache;
export async function loadOptions() {
    const urlOptions = loadUrlOptions(); // used for debugging or preview
    const staticOptions = loadStaticOptions();
    const dynamicOptions = await loadDynamicOptions();
    const localDynamicOptions = await loadLocalDynamicOptions();
    const previewOptions = loadPreviewOptions();
    // languages are to be merged and not replaced
    const languages = {
        ...(staticOptions.languages || {}),
        ...(localDynamicOptions.languages || {}),
        ...(dynamicOptions.languages || {}),
        ...(urlOptions.languages || {})
    };
    let remotes;
    if (urlOptions?.forceUrlRemotes) {
        remotes = getAllMFRemotes([urlOptions]);
    }
    else {
        remotes = getAllMFRemotes([
            staticOptions,
            localDynamicOptions,
            dynamicOptions,
            urlOptions,
            previewOptions
        ]);
    }
    const options = {
        versions: {
            ng1: __VERSION_NG1__,
            ngx: __VERSION_NGX__,
            package: __VERSION_PACKAGE__
        },
        ...staticOptions,
        ...localDynamicOptions,
        ...dynamicOptions,
        ...urlOptions,
        ...previewOptions,
        remotes,
        languages,
        remoteModules: []
    };
    options.C8Y_INSTANCE_OPTIONS = { ...options }; // for compatability with c8yBase.getOptions in ng1-modules
    // If no shades are defined, we auto generate shades. This was implemented with the
    // re-design in 10.17. Branding editor should later define the shades. So long we
    // auto generate shades out of the primary color and based on the referenceShades.
    const shouldApplyShadeColors = hasBrandPrimary(options.brandingCssVars) && !hasAnyBrandShade(options.brandingCssVars);
    if (shouldApplyShadeColors) {
        options.brandingCssVars = getShadeColorBrandingCssVars(options.brandingCssVars);
    }
    return options;
}
function getShadeColorBrandingCssVars(brandingCssVars) {
    const shades = generateShades(brandingCssVars['brand-primary']);
    let i = 1;
    for (const shade of shades) {
        brandingCssVars['c8y-brand-' + i + '0'] = shade;
        i++;
    }
    return brandingCssVars;
}
function generateShades(inputColor) {
    const referenceShades = [
        '#134158',
        '#1C5569',
        '#058192',
        '#22A6AA',
        '#3CC1B7',
        '#8ADBD5',
        '#C5EDEA',
        '#EBF9F8'
    ];
    // Calculate the luminance of the reference shades
    const referenceLuminances = referenceShades.map(color => chroma(color).luminance());
    // Generate shades of the input color with the same luminance as the reference shades
    const generatedShades = referenceLuminances.map(luminance => chroma(inputColor).luminance(luminance).hex());
    // Calculate the distance between the input color and each color in the generatedShades array
    const distances = generatedShades.map(color => chroma.deltaE(inputColor, color));
    // Find the index of the color with the smallest distance
    const index = distances.indexOf(Math.min(...distances));
    generatedShades[index] = inputColor;
    return generatedShades;
}
function hasBrandPrimary(brandingCssVars) {
    return !!brandingCssVars?.['brand-primary'];
}
function hasAnyBrandShade(brandingCssVars) {
    if (!brandingCssVars) {
        return false;
    }
    return !!Object.keys(brandingCssVars).some(value => /brand-[1-8]0/.test(value));
}
export function getAllMFRemotes(options) {
    return reduce(options, (allRemotes, mfRemote) => {
        const { remotes } = mfRemote;
        forEach(remotes, (remoteModules, remoteContextPath) => {
            const currentRemotes = get(allRemotes, remoteContextPath, []);
            allRemotes[remoteContextPath] = union(currentRemotes, remoteModules);
        });
        return allRemotes;
    }, {});
}
function loadStaticOptions() {
    if (!staticOptionsCache) {
        staticOptionsCache =
            JSON.parse(document.querySelector('#static-options').innerText) || {};
    }
    return { ...staticOptionsCache, ...loadUrlOptions() };
}
export async function loginOptions() {
    const hostName = location.origin;
    return await requestRemoteOptions(hostName + '/tenant/loginOptions');
}
export function loadUrlOptions() {
    if (!urlOptionsCache) {
        const query = location.search.substr(1).split('&');
        urlOptionsCache = query.reduce((options, keyValuePair) => {
            if (keyValuePair) {
                if (keyValuePair.match(/=/)) {
                    const [key, value] = keyValuePair.split(/=/);
                    try {
                        options[key] = JSON.parse(decodeURIComponent(value));
                    }
                    catch (error) {
                        console.warn(`Failed to parse option ${key}: ${error}`);
                        options[key] = value;
                    }
                }
                else {
                    options[keyValuePair] = true;
                }
            }
            return options;
        }, {});
    }
    return urlOptionsCache;
}
export function clearUrlOptionsCache() {
    urlOptionsCache = undefined;
}
async function loadLocalDynamicOptions() {
    const remoteOptions = await requestRemoteOptions(`cumulocity.json?nocache=${String(Math.random()).substr(2)}`);
    return remoteOptions;
}
async function loadDynamicOptions() {
    let { dynamicOptionsUrl } = loadStaticOptions();
    let remoteOptions = {};
    if (dynamicOptionsUrl) {
        dynamicOptionsUrl = dynamicOptionsUrl.match(/\?/)
            ? dynamicOptionsUrl
            : `${dynamicOptionsUrl}?nocache=${String(Math.random()).substr(2)}`;
        remoteOptions = await requestRemoteOptions(dynamicOptionsUrl);
    }
    return remoteOptions;
}
function loadPreviewOptions() {
    if (window.C8Y_PREVIEW) {
        return window.C8Y_PREVIEW;
    }
    return {};
}
async function requestRemoteOptions(url) {
    return new Promise((resolve, reject) => {
        const request = new XMLHttpRequest();
        let options = {};
        const onload = () => {
            if (request.status >= 200 && request.status < 400) {
                try {
                    options = JSON.parse(request.responseText);
                }
                catch (e) {
                    // do nothing
                }
                Object.keys(options).forEach(key => {
                    const value = options[key];
                    if (typeof value === 'string') {
                        try {
                            const parsed = JSON.parse(value);
                            if (typeof parsed === 'object') {
                                options[key] = parsed;
                            }
                        }
                        catch (e) {
                            // do nothing
                        }
                    }
                });
            }
            resolve(options);
        };
        request.open('GET', url, true);
        request.setRequestHeader('UseXBasic', 'true');
        request.onload = onload;
        request.onerror = e => reject(e);
        request.send();
    });
}
/**
 * Update the window object with the given options and modules.
 * @param windowObj - The global window object.
 * @param c8yAppVarName - The variable name used for the C8Y_APP object.
 * @param options - An object containing configuration options for the C8Y_APP object.
 * @param modulesToCopy - An array of angularjs modules to be added to the C8Y_APP object.
 * @returns - The updated window object.
 */
function updateWindowObject(windowObj, c8yAppVarName, options) {
    windowObj.C8Y_APP = windowObj[c8yAppVarName] = { modules: [], ...options };
    return windowObj;
}
/**
 * Apply the given options to the C8Y_APP object in the window and update the document elements.
 * @param options - An object containing configuration options for the C8Y_APP object.
 * @returns - The updated options object.
 */
export function applyOptions(options) {
    const c8yAppVarName = options['c8yAppVarName'] || 'C8Y_APP';
    const updatedWindow = updateWindowObject(window, c8yAppVarName, options);
    options = updatedWindow[c8yAppVarName];
    setVersion(options);
    updateTitle(options, document);
    updateFavicon(options, document);
    updateBrandingUrl(options, document);
    updateCss(options, document);
    updateTranslations(options);
    return options;
}
export function setVersion(options) {
    const { c8yVersionName = 'UI_VERSION' } = options;
    window[c8yVersionName] = options.versions.ng1 || options.versions.ngx;
}
export function updateTitle({ globalTitle }, document) {
    if (!globalTitle) {
        return;
    }
    const titleEl = document.querySelector('title');
    titleEl.innerText = `${globalTitle} - ${titleEl.innerText}`;
}
export function updateFavicon({ faviconUrl = 'favicon.ico' }, document) {
    const link = document.createElement('link');
    link.setAttribute('rel', 'icon');
    link.setAttribute('href', faviconUrl);
    document.querySelector('head').appendChild(link);
}
export function updateBrandingUrl({ brandingUrl }, document) {
    if (!brandingUrl) {
        if (__ENTRY_BRANDING__) {
            /**
             * Added ngGlobalStyle. Without it, the Angular webpack configuration only uses less-loader for project styles (c8y/style package).
             * After adding ngGlobalStyle, the rest of the loaders: postcss, css, and mini-css-extract will be applied to the project styles.
             */
            import(/* webpackChunkName: "branding" */ `${__ENTRY_BRANDING__}?ngGlobalStyle`);
        }
        else {
            throw new Error('Branding definition missing.');
        }
    }
    else {
        const link = document.createElement('link');
        link.setAttribute('rel', 'stylesheet');
        link.setAttribute('href', brandingUrl);
        document.querySelector('head').appendChild(link);
    }
}
export function updateCss({ brandingCssVars, extraCssUrls }, document) {
    if (Array.isArray(extraCssUrls)) {
        extraCssUrls.forEach(url => {
            const link = document.createElement('link');
            link.setAttribute('rel', 'stylesheet');
            link.setAttribute('href', url);
            document.querySelector('head').appendChild(link);
        });
    }
    if (brandingCssVars) {
        const vars = Object.keys(brandingCssVars).map(key => `--${key}: ${brandingCssVars[key]};`);
        const style = document.createElement('style');
        style.appendChild(document.createTextNode(`:root{\n${vars.join('\n')}\n}`));
        document.querySelector('body').appendChild(style);
    }
}
export function updateTranslations(options) {
    if (options?.i18nExtra) {
        options.langsDetails = { ...options.langsDetails, ...options.i18nExtra };
    }
}
//# sourceMappingURL=options-resolver.js.map